@@ -49,6 +49,7 @@
 #include "nsIMsgDBView.h"
 #include "nsISupportsObsolete.h"
 #include "nsServiceManagerUtils.h"
+#include "nsImapCore.h"
 
 static const char *kDBFolderInfoScope = "ns:msg:db:row:scope:dbfolderinfo:all";
 static const char *kDBFolderInfoTableKind = "ns:msg:db:table:kind:dbfolderinfo";
@@ -180,13 +181,13 @@ nsDBFolderInfo::nsDBFolderInfo(nsMsgDatabase *mdb)
   // mail only (for now)
   m_folderSize = 0;
   m_folderDate = 0;
-  m_expungedBytes = 0;	// sum of size of deleted messages in folder
+  m_expungedBytes = 0; // sum of size of deleted messages in folder
   m_highWaterMessageKey = 0;
 
   m_numUnreadMessages = 0;
   m_numMessages = 0;
   // IMAP only
-  m_ImapUidValidity = 0;
+  m_ImapUidValidity = kUidUnknown;
   m_totalPendingMessages =0;
   m_unreadPendingMessages = 0;
 
@@ -402,9 +403,9 @@ nsresult nsDBFolderInfo::LoadMemberVariables()
   GetInt32PropertyWithToken(m_numMessagesColumnToken, m_numMessages);
   GetInt32PropertyWithToken(m_numUnreadMessagesColumnToken, m_numUnreadMessages);
   GetInt32PropertyWithToken(m_flagsColumnToken, m_flags);
-  GetInt32PropertyWithToken(m_folderSizeColumnToken, m_folderSize);
+  GetUint64PropertyWithToken(m_folderSizeColumnToken, &m_folderSize);
   GetInt32PropertyWithToken(m_folderDateColumnToken, (PRInt32 &) m_folderDate);
-  GetInt32PropertyWithToken(m_imapUidValidityColumnToken, m_ImapUidValidity);
+  GetInt32PropertyWithToken(m_imapUidValidityColumnToken, m_ImapUidValidity, kUidUnknown);
   GetInt32PropertyWithToken(m_expiredMarkColumnToken, (PRInt32 &) m_expiredMark);
   GetInt32PropertyWithToken(m_expungedBytesColumnToken, (PRInt32 &) m_expungedBytes);
   GetInt32PropertyWithToken(m_highWaterMessageKeyColumnToken, (PRInt32 &) m_highWaterMessageKey);
@@ -457,17 +458,17 @@ NS_IMETHODIMP nsDBFolderInfo::OnKeyAdded(nsMsgKey aNewKey)
 }
 
 NS_IMETHODIMP
-nsDBFolderInfo::GetFolderSize(PRUint32 *size)
+nsDBFolderInfo::GetFolderSize(PRUint64 *size)
 {
   NS_ENSURE_ARG_POINTER(size);
   *size = m_folderSize;
   return NS_OK;
 }
 
-NS_IMETHODIMP	nsDBFolderInfo::SetFolderSize(PRUint32 size)
+NS_IMETHODIMP nsDBFolderInfo::SetFolderSize(PRUint64 size)
 {
   m_folderSize = size;
-  return SetUint32PropertyWithToken(m_folderSizeColumnToken, m_folderSize);
+  return SetUint64Property(kFolderSizeColumnName, m_folderSize);
 }
 
 NS_IMETHODIMP
@@ -486,6 +487,35 @@ NS_IMETHODIMP nsDBFolderInfo::SetFolderDate(PRUint32 folderDate)
 
 NS_IMETHODIMP nsDBFolderInfo::GetHighWater(nsMsgKey *result)
 {
+  // Sanity check highwater - if it gets too big, other code
+  // can fail.
+  *result = m_highWaterMessageKey;
+  if (m_highWaterMessageKey > 0xFFFFFF00 && m_mdb)
+  {
+    nsCOMPtr <nsISimpleEnumerator> hdrs;
+    nsresult rv = m_mdb->ReverseEnumerateMessages(getter_AddRefs(hdrs));
+    if (NS_FAILED(rv))
+      return rv;
+    PRBool hasMore = PR_FALSE;
+    nsCOMPtr<nsIMsgDBHdr> pHeader;
+    nsMsgKey recalculatedHighWater = 1;
+    PRInt32 i = 0;
+    while(i++ < 100 && NS_SUCCEEDED(rv = hdrs->HasMoreElements(&hasMore))
+              && hasMore)
+    {
+      rv = hdrs->GetNext(getter_AddRefs(pHeader));
+      if (pHeader)
+      {
+        nsMsgKey msgKey;
+        pHeader->GetMessageKey(&msgKey);
+        if (msgKey > recalculatedHighWater)
+          recalculatedHighWater = msgKey;
+      }
+    }
+    NS_ASSERTION(m_highWaterMessageKey >= recalculatedHighWater,
+                 "highwater incorrect");
+    m_highWaterMessageKey = recalculatedHighWater;
+  }
   *result = m_highWaterMessageKey;
   return NS_OK;
 }
@@ -666,6 +696,7 @@ NS_IMETHODIMP nsDBFolderInfo::SetCharacterSet(const nsACString &charSet)
 
 NS_IMETHODIMP nsDBFolderInfo::GetCharacterSetOverride(PRBool *characterSetOverride)
 {
+  NS_ENSURE_ARG_POINTER(characterSetOverride);
   *characterSetOverride = m_charSetOverride;
   return NS_OK;
 }
@@ -846,6 +877,12 @@ nsresult  nsDBFolderInfo::SetUint32PropertyWithToken(mdb_token aProperty, PRUint
   return m_mdb->UInt32ToRowCellColumn(m_mdbRow, aProperty, propertyValue);
 }
 
+nsresult  nsDBFolderInfo::SetUint64Property(const char *aProperty,
+                                            PRUint64 propertyValue)
+{
+  return m_mdb->SetUint64Property(m_mdbRow, aProperty, propertyValue);
+}
+
 nsresult  nsDBFolderInfo::SetInt32PropertyWithToken(mdb_token aProperty, PRInt32 propertyValue)
 {
   nsAutoString propertyStr;
@@ -873,6 +910,12 @@ NS_IMETHODIMP nsDBFolderInfo::GetUint32Property(const char *propertyName, PRUint
   return m_mdb->GetUint32Property(m_mdbRow, propertyName, propertyValue, defaultValue);
 }
 
+nsresult nsDBFolderInfo::GetUint64PropertyWithToken(mdb_token columnToken,
+                                                    PRUint64 *propertyValue)
+{
+  return m_mdb->RowCellColumnToUInt64(m_mdbRow, columnToken, propertyValue, 0);
+}
+
 NS_IMETHODIMP nsDBFolderInfo::GetBooleanProperty(const char *propertyName, PRBool defaultValue, PRBool *propertyValue)
 {
   PRUint32 defaultUint32Value = (defaultValue) ? 1 : 0;
