@@ -212,7 +212,7 @@ struct nsPlaybackRequest
   {
   }
   nsImapMailFolder *SrcFolder;
-  nsIMsgWindow *MsgWindow;
+  nsCOMPtr<nsIMsgWindow> MsgWindow;
 };
 
 class nsImapMailFolder :  public nsMsgDBFolder,
@@ -317,6 +317,8 @@ public:
   NS_IMETHOD AddKeywordsToMessages(nsIArray *aMessages, const nsACString& aKeywords);
   NS_IMETHOD RemoveKeywordsFromMessages(nsIArray *aMessages, const nsACString& aKeywords);
 
+  NS_IMETHOD NotifyCompactCompleted();
+
   NS_DECL_NSIMSGIMAPMAILFOLDER
   NS_DECL_NSIIMAPMAILFOLDERSINK
   NS_DECL_NSIIMAPMESSAGESINK
@@ -351,6 +353,9 @@ public:
   nsresult SetSupportedUserFlags(PRUint32 userFlags);
   nsresult GetSupportedUserFlags(PRUint32 *userFlags);
 
+  // Find the start of a range of msgKeys that can hold srcCount headers.
+  nsresult FindOpenRange(nsMsgKey &fakeBase, PRUint32 srcCount);
+
 protected:
   // Helper methods
 
@@ -374,9 +379,13 @@ protected:
   nsresult  NormalEndHeaderParseStream(nsIImapProtocol *aProtocol, nsIImapUrl *imapUrl);
 
   void EndOfflineDownload();
+  nsresult CopyFileToOfflineStore(nsILocalFile *srcFile);
 
   nsresult MarkMessagesImapDeleted(nsTArray<nsMsgKey> *keyArray, PRBool deleted, nsIMsgDatabase *db);
 
+  // Notifies imap autosync that it should update this folder when it
+  // gets a chance.
+  void NotifyHasPendingMsgs();
   void UpdatePendingCounts();
   void SetIMAPDeletedFlag(nsIMsgDatabase *mailDB, const nsTArray<nsMsgKey> &msgids, PRBool markDeleted);
   virtual PRBool ShowDeletedMessages();
@@ -450,7 +459,6 @@ protected:
 
   PRBool m_initialized;
   PRBool m_haveDiscoveredAllFolders;
-  PRBool m_haveReadNameFromDB;
   nsCOMPtr<nsIMsgParseMailMsgState> m_msgParser;
   nsCOMPtr<nsIMsgFilterList> m_filterList;
   nsCOMPtr<nsIMsgFilterPlugin> m_filterPlugin;  // XXX should be a list
@@ -464,8 +472,6 @@ protected:
   nsCOMPtr<nsIMsgFolder> mSpamFolder;
   nsMsgKey m_curMsgUid;
   PRUint32 m_uidValidity;
-  // used for condstore support;
-  PRUint64 m_highestModSeq;
 
   // These three vars are used to store counts from STATUS or SELECT command
   // They include deleted messages, so they can differ from the generic
@@ -484,7 +490,6 @@ protected:
   // *** jt - undo move/copy trasaction support
   nsRefPtr<nsMsgTxn> m_pendingUndoTxn;
   nsCOMPtr<nsImapMailCopyState> m_copyState;
-  PRMonitor *m_appendMsgMonitor;
   char m_hierarchyDelimiter;
   PRInt32 m_boxFlags;
   nsCString m_onlineFolderName;
@@ -503,29 +508,33 @@ protected:
   PRPackedBool m_folderQuotaCommandIssued;
   PRPackedBool m_folderQuotaDataIsValid;
   PRPackedBool m_updatingFolder;
+  // These two vars are used to keep track of compaction state so we can know
+  // when to send a done notification.
+  PRPackedBool m_compactingOfflineStore;
+  PRPackedBool m_expunging;
   PRPackedBool m_applyIncomingFilters; // apply filters to this folder, even if not the inbox
   nsMsgIMAPFolderACL *m_folderACL;
   PRUint32     m_aclFlags;
   PRUint32     m_supportedUserFlags;
 
-  static nsIAtom* mImapHdrDownloadedAtom;
-
   // offline imap support
   PRBool m_downloadingFolderForOfflineUse;
   PRBool m_filterListRequiresBody;
-  
+
   // auto-sync (preemptive download) support
   nsRefPtr<nsAutoSyncState> m_autoSyncStateObj;
-  
+
   // Quota support
   nsCString m_folderQuotaRoot;
   PRUint32 m_folderQuotaUsedKB;
   PRUint32 m_folderQuotaMaxKB;
-  
+
   // Pseudo-Offline Playback support
   nsPlaybackRequest *m_pendingPlaybackReq;
   nsCOMPtr<nsITimer> m_playbackTimer;
   nsTArray<nsRefPtr<nsImapMoveCopyMsgTxn> > m_pendingOfflineMoves;
-
+  // hash table of mapping between messageids and message keys
+  // for pseudo hdrs.
+  nsDataHashtable<nsCStringHashKey, nsMsgKey> m_pseudoHdrs;
 };
 #endif
